/**
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 *
 *  Authors:
 *  2009-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     Mischa Sall\'e <msalle@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl> 
 *
 *  2007-2009
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *
 *  2003-2007
 *     Martijn Steenbakkers <martijn@nikhef.nl>
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include <dlfcn.h>

#include <lcas/lcas.h>

#include "utils.h"


#ifndef LIBLCAS_SO
    #define LIBLCAS_SO "liblcas.so"
#endif


#ifndef USE_GLOBUS_INTERFACE_TO_LCMAPS /* Make use of the Globus interface to LCMAPS */

/* AUTHORISATION */
int run_lcas (char * pem_string, FILE * logfile)
{
    int    retval  = 0;
    char * error   = NULL;
#ifndef LINKED_LCAS_LCMAPS
    void * lcas_handle = NULL;
    int (*LcasInit)(FILE *);
    int (*LcasPem)(char *, lcas_request_t);
    int (*LcasTerm)();
#endif


    /* If there is no PEM String, nothing to do */
    if (!pem_string)
    {
        logmsg(LOG_ERR, "Couldn't extract the client certificate credentials.");
        return 1;
    }


#ifndef LINKED_LCAS_LCMAPS
    lcas_handle = dlopen(LIBLCAS_SO, RTLD_LAZY|RTLD_GLOBAL);
    if (!lcas_handle)
    {
        logmsg(LOG_ERR, "Failed to dynamically load the library for LCAS: \"%s\"\n", LIBLCAS_SO);
        return 1;
    }

    /* Check the symbols */
    LcasInit=dlsym(lcas_handle,"lcas_init");
    if ((error = dlerror()) != NULL)
    {
        logmsg(LOG_ERR, "LCAS module not compliant: Symbol \"lcas_init\" not found: %s", error);
        dlclose(lcas_handle);
        return 1;
    }
    LcasPem=dlsym(lcas_handle,"lcas_pem");
    if ((error = dlerror()) != NULL)
    {
        logmsg(LOG_ERR, "LCAS module not compliant: Symbol \"lcas_pem\" not found: %s", error);
        dlclose(lcas_handle);
        return 1;
    }
    LcasTerm=dlsym(lcas_handle,"lcas_term");
    if ((error = dlerror()) != NULL)
    {
        logmsg(LOG_ERR, "LCAS module not compliant: Symbol \"lcas_term\" not found: %s", error);
        dlclose(lcas_handle);
        return 1;
    }
#endif


    /* Initialize, send authorization request to and terminate the LCAS */
    /* retval=lcas_init(config->usrlog_fp); */
#ifdef LINKED_LCAS_LCMAPS
    if ((retval=lcas_init(logfile)))
    {
        logmsg(LOG_ERR, "LCAS initialization failure.");
		return 1;
    }
#else
    if ((retval = (*LcasInit)(logfile)))
    {
        logmsg(LOG_ERR, "LCAS initialization failure.");
        dlclose(lcas_handle);
		return 1;
    }
#endif


    /* Authorization based on the PEM string input, with a faked RSL string, because this interface doesn't pass it on (yet) */
    /* if (lcas_pem(pem_string, request) != 0) */
#ifdef LINKED_LCAS_LCMAPS
    if ((retval = lcas_pem(pem_string, "RSL")) != 0)
#else
    if ((retval = (*LcasPem)(pem_string, "RSL")) != 0)
#endif
    {
        logmsg(LOG_WARNING, "%s: The user is not authorized by LCAS", __func__);
    }
    else
    {
        logmsg(LOG_WARNING, "%s: The user is authorized by LCAS", __func__);
    }

    /* Terminate LCAS framework */
#ifdef LINKED_LCAS_LCMAPS
    if (lcas_term())
    {
        logmsg(LOG_ERR, "LCAS termination failure.");
		return 1;
    }
#else
    if ((*LcasTerm)())
    {
        logmsg(LOG_ERR, "LCAS termination failure.");
        dlclose(lcas_handle);
		return 1;
    }

    /* Release the library handle */
    dlclose(lcas_handle);
#endif

    /* All is ok - but you could still be not authorized! */
    return retval;
}

/***************************************************************************************/
#else /* USE_GLOBUS_INTERFACE_TO_LCMAPS  :  Make use of the Globus interface to LCMAPS */
/***************************************************************************************/


/* AUTHORISATION WITH GLOBUS INTERFACE */
int run_lcas(gss_cred_id_t user_cred_handle, char * client_name, FILE * logfile)
{
    int    retval  = 0;
    char * error   = NULL;
#ifndef LINKED_LCAS_LCMAPS
    void * lcas_handle = NULL;
    int (*LcasInit)(FILE *);
    int (*LcasGetFabricAuthorization)(char*, gss_cred_id_t, char*);
    int (*LcasTerm)();
#endif


    /* If there is no PEM String, nothing to do */
    if (!user_cred_handle)
    {
        logmsg(LOG_ERR, "Couldn't extract the client certificate credentials.");
        return 1;
    }


#ifndef LINKED_LCAS_LCMAPS
    lcas_handle = dlopen(LIBLCAS_SO, RTLD_LAZY|RTLD_GLOBAL);
    if (!lcas_handle)
    {
        logmsg(LOG_ERR, "Failed to dynamically load the library for LCAS: \"%s\"\n", LIBLCAS_SO);
        return 1;
    }

    /* Check the symbols */
    LcasInit=dlsym(lcas_handle,"lcas_init");
    if ((error = dlerror()) != NULL)
    {
        logmsg(LOG_ERR, "LCAS module not compliant: Symbol \"lcas_init\" not found: %s", error);
        dlclose(lcas_handle);
        return 1;
    }
    LcasGetFabricAuthorization=dlsym(lcas_handle,"lcas_get_fabric_authorization");
    if ((error = dlerror()) != NULL)
    {
        logmsg(LOG_ERR, "LCAS module not compliant: Symbol \"lcas_get_fabric_authorization\" not found: %s", error);
        dlclose(lcas_handle);
        return 1;
    }
    LcasTerm=dlsym(lcas_handle,"lcas_term");
    if ((error = dlerror()) != NULL)
    {
        logmsg(LOG_ERR, "LCAS module not compliant: Symbol \"lcas_term\" not found: %s", error);
        dlclose(lcas_handle);
        return 1;
    }
#endif


    /* Initialize, send authorization request to and terminate the LCAS */
    /* retval=lcas_init(config->usrlog_fp); */
#ifdef LINKED_LCAS_LCMAPS
    if ((retval=lcas_init(logfile)))
    {
        logmsg(LOG_ERR, "LCAS initialization failure.");
		return 1;
    }
#else
    if ((retval = (*LcasInit)(logfile)))
    {
        logmsg(LOG_ERR, "LCAS initialization failure.");
        dlclose(lcas_handle);
		return 1;
    }
#endif


    /* Authorization based on the GSI input */
#ifdef LINKED_LCAS_LCMAPS
    if ((retval=lcas_get_fabric_authorization(client_name, user_cred_handle, "RSL") != 0))
#else
    if ((retval=(*LcasGetFabricAuthorization)(client_name, user_cred_handle, "RSL") != 0))
#endif
    {
        logmsg(LOG_WARNING, "%s: The user is not authorized by LCAS", __func__);
    }
    else
    {
        logmsg(LOG_WARNING, "%s: The user is authorized by LCAS", __func__);
    }

    /* Terminate LCAS framework */
#ifdef LINKED_LCAS_LCMAPS
    if (lcas_term())
    {
        logmsg(LOG_ERR, "LCAS termination failure.");
		return 1;
    }
#else
    if ((*LcasTerm)())
    {
        logmsg(LOG_ERR, "LCAS termination failure.");
        dlclose(lcas_handle);
		return 1;
    }

    /* Release the library handle */
    dlclose(lcas_handle);
#endif

    /* All is ok - but you could still be not authorized! */
    return retval;
}

#endif /* USE_GLOBUS_INTERFACE_TO_LCMAPS  :  Make use of the Globus interface to LCMAPS */
